#ifndef TWN_UTIL_H
#define TWN_UTIL_H

#include "twn_types.h"
#include "twn_engine_api.h"

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>


#ifndef TWN_NOT_C
    #include <math.h>

    #ifndef M_PI
    #define M_PI 3.14159265358979323846264338327950288   /**< pi */
    #endif

    /* multiply by these to convert degrees <---> radians */
    #define DEG2RAD (M_PI / 180)
    #define RAD2DEG (180 / M_PI)

    TWN_API void log_info(const char *restrict format, ...);
    TWN_API void log_critical(const char *restrict format, ...);
    TWN_API void log_warn(const char *restrict format, ...);

    /* saves all texture atlases as BMP files in the write directory */
    TWN_API void textures_dump_atlases(void);

    /* TODO: this is why generics were invented. sorry, i'm tired today */
    TWN_API float clampf(float f, float min, float max);

    /* sets buf_out to a pointer to a byte buffer which must be freed. */
    /* returns the size of this buffer. */
    TWN_API int64_t file_to_bytes(const char *path, unsigned char **buf_out);

    /* returns a pointer to a string which must be freed */
    TWN_API char *file_to_str(const char *path);

    /* returns true if the file exists in the filesystem */
    TWN_API bool file_exists(const char *path);

#endif /* TWN_NOT_C */

/* read file to null terminated string, it is freed when the frame ends */
TWN_API char const *file_read(char const *file);

/* calculates the overlap of two rectangles */
TWN_API Rect rect_overlap(Rect a, Rect b);
/* returns true if two rectangles are intersecting */
TWN_API bool rect_intersects(Rect a, Rect b);

/* decrements a floating point second-based timer, stopping at 0.0f */
/* meant for poll based real time logic in game logic */
/* note that it should be decremented only on the next tick after its creation */
TWN_API float timer_tick_seconds(float seconds_left);

typedef struct TimerElapseSecondsResult {
    float seconds_left; float interval; bool elapsed;
} TimerElapseSecondsResult;
TWN_API TimerElapseSecondsResult timer_elapse_seconds(float seconds_left, float interval);

TWN_API void log_vec2(Vec2 value, char const *identity);
TWN_API void log_vec3(Vec3 value, char const *identity);
TWN_API void log_rect(Rect value, char const *identity);

TWN_API void profile_start(char const *profile);
TWN_API void profile_end(char const *profile);

#endif
